# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-22 14:13
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('thorbanks', '__first__'),
    ]

    operations = [
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.FloatField()),
                ('is_paid', models.BooleanField(default=False)),
                ('transaction', models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, to='thorbanks.Transaction')),
            ],
        ),
    ]



from __future__ import unicode_literals

import os

from selenium.common.exceptions import NoSuchElementException

IPIZZA_BANKS = [
    'swedbank',
    'seb',
    'lhv',
    'danske',
]

IS_TRAVIS = os.environ.get('TRAVIS', None)
TIMEOUT = 200 if IS_TRAVIS else 20


def click(selector, selenium):
    element = selenium.find_element_by_css_selector(selector)

    assert element and (not isinstance(element, list) or len(element) == 1)

    if isinstance(element, list):
        element = element[0]

    element.click()

    return element


def select_radio(name, value, selenium):
    click('input[name="%s"][value="%s"]' % (name, value), selenium)
    checked = selenium.find_element_by_css_selector('input[name="%s"]:checked' % name)

    assert checked
    assert checked.get_attribute('name') == name
    assert checked.get_attribute('value') == value


def set_input_text(name, value, selenium):
    element = click('input[name="%s"]' % name, selenium)

    element.clear()
    element.send_keys(value)
    element = selenium.find_element_by_css_selector('input[name="%s"]' % name)

    assert element
    assert element.get_attribute('name') == name
    assert element.get_attribute('value') == value


def ready(d):
    return d.execute_script("return document.readyState") == "complete"


def assert_no_errors(selenium, err_type='payment'):
    try:
        err = selenium.find_element_by_css_selector("[data-%s-error]" % err_type)

    except NoSuchElementException:
        pass

    else:
        raise AssertionError("Creating payment error: %s" % (err.text or err.get_attribute('data-payment-error')))


def assert_ex_msg(e, expected):
    if hasattr(e, 'value'):
        if isinstance(e.value, Exception):
            e = e.value

    message = getattr(e, 'message', getattr(e, 'args', ''))

    if isinstance(message, (list, tuple)):
        if message:
            message = message[0]

        else:
            message = ''

    assert message == expected



"""
Django settings for example project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""

from __future__ import unicode_literals

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
import django

BASE_DIR = os.path.dirname(__file__)


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '-zta41a7+#1v=uv30j%9dxtbfh9*#=dt6l(m9$ce8p20p_=n@s'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATE_DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'thorbanks',

    'shop',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'example.urls'

WSGI_APPLICATION = 'example.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_URL = '/static/'

TEMPLATE_DIRS = [
    os.path.join(BASE_DIR, 'templates'),
]

if django.VERSION[0] == 1 and django.VERSION[1] <= 4:
    TEMPLATE_DIRS = [
        os.path.join(BASE_DIR, 'legacy_templates'),
    ] + TEMPLATE_DIRS


# Bank links
BANKLINKS = {
    'swedbank': {
        'PRINTABLE_NAME': 'Swedbank',
        'REQUEST_URL': 'http://banks.maximum.thorgate.eu/banklink/swedbank-common',
        'CLIENT_ID': 'uid100052',
        'BANK_ID': 'HP',
        'PRIVATE_KEY': os.path.join(BASE_DIR, '..', 'certs', 'swed_key.pem'),
        'PUBLIC_KEY': os.path.join(BASE_DIR, '..', 'certs', 'swed_pub.pem'),
        'TYPE': 'banklink',
        'IMAGE_PATH': 'swedbank.png',
        'ORDER': 1,
    },
    'seb': {
        'PRINTABLE_NAME': 'SEB',
        'REQUEST_URL': 'http://banks.maximum.thorgate.eu/banklink/seb-common',
        'CLIENT_ID': 'uid100036',
        'BANK_ID': 'EYP',
        'PRIVATE_KEY': os.path.join(BASE_DIR, '..', 'certs', 'seb_key.pem'),
        'PUBLIC_KEY': os.path.join(BASE_DIR, '..', 'certs', 'seb_pub.pem'),
        'DIGEST_COUNTS_BYTES': True,
        'TYPE': 'banklink',
        'IMAGE_PATH': 'seb.png',
        'ORDER': 2,
    },
    'danske': {
        'PRINTABLE_NAME': 'Danske Bank',
        'REQUEST_URL': 'http://banks.maximum.thorgate.eu/banklink/sampo-common',
        'CLIENT_ID': 'uid100010',
        'BANK_ID': 'SAMPOPANK',
        'PRIVATE_KEY': os.path.join(BASE_DIR, '..', 'certs', 'danske_key.pem'),
        'PUBLIC_KEY': os.path.join(BASE_DIR, '..', 'certs', 'danske_pub.pem'),
        'ENCODING': 'ISO-8859-1',
        'TYPE': 'banklink',
        'IMAGE_PATH': 'danske.png',
        'ORDER': 3,
    },
    'nordea': {
        'PRINTABLE_NAME': 'Nordea',
        'REQUEST_URL': 'https://netbank.nordea.com/pnbeidtest/eid.jsp',
        'CLIENT_ID': '87654321',
        'MAC_KEY': 'LEHTI',
        'TYPE': 'banklink',
        'PROTOCOL': 'nordea',
        'ORDER': 4,
    },
    'lhv': {
        'PRINTABLE_NAME': 'LHV',
        'REQUEST_URL': 'http://banks.maximum.thorgate.eu/banklink/lhv-common',
        'CLIENT_ID': 'uid100049',
        'BANK_ID': 'LHV',
        'PRIVATE_KEY': os.path.join(BASE_DIR, '..', 'certs', 'lhv_key.pem'),
        'PUBLIC_KEY': os.path.join(BASE_DIR, '..', 'certs', 'lhv_pub.pem'),
        'TYPE': 'banklink',
        'IMAGE_PATH': 'lhv.png',
        'ORDER': 5,
    },
}
# Here you can customize where the bank logos are (used by the PaymentFormMixin). This is relative to STATIC_URL and
#  must end with slash
# BANKLINK_LOGO_PATH



from __future__ import unicode_literals

import os
import sys


BASE_DIR = os.path.dirname(__file__)

sys.path.append(os.path.join(BASE_DIR, '..'))
sys.path.append(os.path.join(BASE_DIR, '..', 'example'))



from __future__ import unicode_literals

import django.dispatch


transaction_started = django.dispatch.Signal(providing_args=['transaction'])
transaction_succeeded = django.dispatch.Signal(providing_args=['transaction'])
transaction_failed = django.dispatch.Signal(providing_args=['transaction'])

auth_started = django.dispatch.Signal(providing_args=['auth'])
auth_succeeded = django.dispatch.Signal(providing_args=['auth'])
auth_failed = django.dispatch.Signal(providing_args=['auth'])



from __future__ import unicode_literals

import os

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from .loading import get_model as _get_model


def _configure():
    links = getattr(settings, 'BANKLINKS', None)

    if not links:
        raise ImproperlyConfigured(u"BANKLINKS not found in settings")

    elif not isinstance(links, dict):
        raise ImproperlyConfigured(u"settings.BANKLINKS must be dict")

    else:
        res = {}

        # Verify it contains proper data
        for bank_name, data in links.items():
            if len(bank_name) > 16:
                raise ImproperlyConfigured(u"Bank's name must be at most 16 characters (%s)" % bank_name)

            if not isinstance(data, dict):
                raise ImproperlyConfigured(u"Each bank in settings.BANKLINKS must correspond to dict with settings of that bank")

            if 'REQUEST_URL' not in data:
                raise ImproperlyConfigured(u"REQUEST_URL not found in settings for bank %s" % bank_name)

            protocol = data.get('PROTOCOL', 'ipizza')

            if protocol == 'ipizza':
                if 'PRIVATE_KEY' not in data:
                    raise ImproperlyConfigured(u"PRIVATE_KEY not found in settings for bank %s" % bank_name)

                if 'PUBLIC_KEY' not in data:
                    raise ImproperlyConfigured(u"PUBLIC_KEY not found in settings for bank %s" % bank_name)

                if 'SND_ID' in data:
                    raise ImproperlyConfigured(u"SND_ID is removed. Use CLIENT_ID instead %s" % bank_name)

                if 'CLIENT_ID' not in data:
                    raise ImproperlyConfigured(u"CLIENT_ID not found in settings for bank %s" % bank_name)

                if 'BANK_ID' not in data:
                    raise ImproperlyConfigured(u"BANK_ID not found in settings for bank %s" % bank_name)

                if not os.path.isfile(data['PRIVATE_KEY']):
                    raise ImproperlyConfigured(u"Private key file %s for bank %s does not exist." % (data['PRIVATE_KEY'], bank_name))

                if not os.path.isfile(data['PUBLIC_KEY']):
                    raise ImproperlyConfigured(u"Public key file %s for bank %s does not exist." % (data['PUBLIC_KEY'], bank_name))

                data['PUBLIC_KEY'] = os.path.abspath(data['PUBLIC_KEY'])
                data['PRIVATE_KEY'] = os.path.abspath(data['PRIVATE_KEY'])

            elif protocol == 'nordea':
                if 'MAC_KEY' not in data:
                    raise ImproperlyConfigured(u"MAC_KEY not found in settings for bank %s" % bank_name)

            res[bank_name] = data

    return res


def get_model(model_name):
    manual = getattr(settings, 'THORBANKS_MANUAL_MODELS', {})
    model_full_name = manual.get(model_name, 'thorbanks.%s' % model_name)
    return _get_model(model_full_name)


def manual_models(model_name):
    manual = getattr(settings, 'THORBANKS_MANUAL_MODELS', {})

    if isinstance(manual, (list, tuple)):
        raise ImproperlyConfigured("THORBANKS_MANUAL_MODELS setting should now be dict")

    return model_name in manual


# Shorthand methods
def get_private_key(the_bank):
    return LINKS[the_bank]['PRIVATE_KEY']


def get_public_key(the_bank):
    return LINKS[the_bank]['PUBLIC_KEY']


def get_client_id(the_bank):
    return LINKS[the_bank]['CLIENT_ID']


def get_bank_id(the_bank):
    return LINKS[the_bank]['BANK_ID']


def get_request_url(the_bank):
    return LINKS[the_bank]['REQUEST_URL']


def get_link_type(the_bank):
    return LINKS[the_bank]['TYPE']


def get_link_protocol(the_bank):
    return LINKS[the_bank].get('PROTOCOL', 'ipizza')


def get_send_ref(the_bank):
    return LINKS[the_bank].get('SEND_REF', True)


def get_bank_choices():
    """ Returns list of (bank_name, pretty_name, image_path, order) tuples.
    Useful in forms
    """
    return [(the_bank,
             LINKS[the_bank].get('PRINTABLE_NAME', the_bank),
             LINKS[the_bank].get('IMAGE_PATH', the_bank),
             LINKS[the_bank].get('ORDER', 99)) for the_bank in LINKS.keys()]


LINKS = {}


def configure():
    global LINKS

    LINKS = _configure()


configure()



from __future__ import unicode_literals

from warnings import warn

from django import forms
from django.conf import settings as django_settings
from django.forms.widgets import RadioFieldRenderer
from django.http import HttpResponse
from django.template.loader import render_to_string
from django.utils.encoding import force_text
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from thorbanks import settings
from thorbanks.settings import get_send_ref
from thorbanks.utils import create_signature, nordea_generate_mac
from thorbanks.signals import transaction_started
from thorbanks.utils import calculate_731_checksum


class AuthRequestBase(forms.Form):
    def __init__(self, bank_name, redirect_to, response_url, *args, **kwargs):
        self.auth = settings.get_model('Authentication')()
        self.auth.bank_name = bank_name
        self.auth.redirect_after_success = redirect_to
        self.auth.redirect_on_failure = redirect_to
        self.auth.save()

        initial = self.prepare(bank_name, redirect_to, response_url, *args, **kwargs)

        super(AuthRequestBase, self).__init__(initial, *args, **kwargs)

        if not self.is_valid():
            raise RuntimeError("invalid initial data")  # pragma no cover

        self.finalize()
        if not self.is_valid():
            raise RuntimeError("signature is invalid")  # pragma no cover

    def prepare(self, *args, **kwargs):
        raise NotImplementedError  # pragma no cover

    def finalize(self):
        raise NotImplementedError  # pragma no cover

    def redirect_html(self):
        """ Redirection html """
        html = '<form action="%s" method="POST" id="banklink_redirect_url" accept-charset="%s">' % (
            self.get_request_url(), self.get_encoding())

        for field in self:
            html += force_text(field) + "\n"

        html += '</form>'
        html += """<script type="text/javascript">
                       document.forms['banklink_redirect_url'].submit();
                   </script>
                """

        return mark_safe(html)

    def get_request_url(self):
        return settings.get_request_url(self.auth.bank_name)

    def get_encoding(self):
        return 'UTF-8'


class IPizzaAuthRequest(AuthRequestBase):
    VK_SERVICE = forms.CharField(widget=forms.HiddenInput())
    VK_VERSION = forms.CharField(widget=forms.HiddenInput())
    VK_SND_ID = forms.CharField(widget=forms.HiddenInput())
    VK_REC_ID = forms.CharField(widget=forms.HiddenInput())
    VK_NONCE = forms.CharField(widget=forms.HiddenInput(), max_length=50)
    VK_RETURN = forms.CharField(widget=forms.HiddenInput())
    VK_DATETIME = forms.CharField(widget=forms.HiddenInput())

    VK_ENCODING = forms.CharField(widget=forms.HiddenInput())

    VK_RID = forms.CharField(widget=forms.HiddenInput(), required=False)
    VK_MAC = forms.CharField(widget=forms.HiddenInput(), required=False)

    def prepare(self, bank_name, redirect_to, response_url, *args, **kwargs):
        initial = {
            'VK_SERVICE': '4012',
            'VK_VERSION': '008',
            'VK_SND_ID': settings.get_client_id(bank_name),
            'VK_REC_ID': settings.get_bank_id(bank_name),
            'VK_NONCE': self.auth.pk,
            'VK_RETURN': response_url,
            'VK_DATETIME': self.auth.created.strftime('%Y-%m-%dT%H:%M:%S%z'),
            'VK_RID': '',
            'VK_REPLY': '3013',
            'VK_ENCODING': self.get_encoding(),
        }
        return initial

    def finalize(self):
        mac = create_signature(self.cleaned_data, self.auth.bank_name, auth=True)
        self.data['VK_MAC'] = mac


class NordeaAuthRequest(AuthRequestBase):
    A01Y_ACTION_ID = forms.CharField(widget=forms.HiddenInput())
    A01Y_VERS = forms.CharField(widget=forms.HiddenInput())
    A01Y_RCVID = forms.CharField(widget=forms.HiddenInput())
    A01Y_LANGCODE = forms.CharField(widget=forms.HiddenInput())
    A01Y_STAMP = forms.CharField(widget=forms.HiddenInput())
    A01Y_IDTYPE = forms.CharField(widget=forms.HiddenInput())
    A01Y_RETLINK = forms.CharField(widget=forms.HiddenInput())
    A01Y_CANLINK = forms.CharField(widget=forms.HiddenInput())
    A01Y_REJLINK = forms.CharField(widget=forms.HiddenInput())
    A01Y_KEYVERS = forms.CharField(widget=forms.HiddenInput())
    A01Y_ALG = forms.CharField(widget=forms.HiddenInput())
    A01Y_MAC = forms.CharField(widget=forms.HiddenInput(), required=False)

    def prepare(self, bank_name, redirect_to, response_url, *args, **kwargs):
        initial = {
            'A01Y_ACTION_ID': '701',
            'A01Y_VERS': '0002',
            'A01Y_RCVID': settings.get_client_id(bank_name),
            'A01Y_LANGCODE': 'ET',
            'A01Y_STAMP': self.auth.pk,
            'A01Y_IDTYPE': '02',
            'A01Y_RETLINK': response_url,
            'A01Y_CANLINK': response_url,
            'A01Y_REJLINK': response_url,
            'A01Y_KEYVERS': '0001',     # TODO: support multiple keys
            'A01Y_ALG': '01',           # We use MD5 to calculate MAC since nordea LIVE does not like SHA1
        }
        return initial

    def finalize(self):
        mac_token_fields = (
            'A01Y_ACTION_ID',
            'A01Y_VERS',
            'A01Y_RCVID',
            'A01Y_LANGCODE',
            'A01Y_STAMP',
            'A01Y_IDTYPE',
            'A01Y_RETLINK',
            'A01Y_CANLINK',
            'A01Y_REJLINK',
            'A01Y_KEYVERS',
            'A01Y_ALG',
        )
        link_config = settings.LINKS[self.auth.bank_name]
        mac_key = link_config['MAC_KEY']
        mac = nordea_generate_mac(self.cleaned_data, mac_token_fields, mac_key)

        self.data['A01Y_MAC'] = mac


class PaymentRequestBase(forms.Form):
    def __init__(self, *args, **kwargs):
        self.transaction = settings.get_model('Transaction')()
        self.transaction.bank_name = kwargs['bank_name']
        self.transaction.description = kwargs['message']
        self.transaction.amount = round(kwargs['amount'], 2)
        self.transaction.currency = kwargs['currency']
        self.transaction.message = kwargs['message']

        self.transaction.redirect_after_success = kwargs['redirect_to']
        self.transaction.redirect_on_failure = kwargs['redirect_on_failure']
        self.transaction.save()

        transaction_started.send(settings.get_model('Transaction'), transaction=self.transaction)

        initial = self.prepare(self.transaction, kwargs['url'])

        super(PaymentRequestBase, self).__init__(initial, *args)

        if not self.is_valid():
            raise RuntimeError("invalid initial data")

        self.finalize()
        if not self.is_valid():
            raise RuntimeError("signature is invalid")

    @classmethod
    def prepare(cls, transaction, url, language='EST'):
        raise NotImplementedError

    def finalize(self):
        raise NotImplementedError

    def redirect_html(self):
        html = '<form action="%s" method="POST" id="banklink_redirect_url" accept-charset="%s">' % (
            self.get_request_url(), self.get_encoding())
        for field in self:
            html += force_text(field) + u"\n"
        html += '</form>'
        html += '''<script type="text/javascript">
                    document.forms['banklink_redirect_url'].submit();
                    </script>'''
        return mark_safe(html)

    def submit_button(self, value=u"Make the payment"):
        html = '<form action="%s" method="POST">' % (settings.get_request_url(self.transaction.bank_name))
        for field in self:
            html += force_text(field) + u"\n"
        html += '<input type="submit" value="%s" />' % value
        html += '</form>'
        return mark_safe(html)

    def as_html(self, with_submit=False, id="banklink_payment_form", submit_value="submit"):
        warn("deprecated", DeprecationWarning)
        html = '<form action="%s" method="POST" id="%s">' % (settings.get_request_url(self.transaction.bank_name), id)
        for field in self:
            html += force_text(field) + u"\n"
        if with_submit:
            html += '<input type="submit" value="%s"/>' % (submit_value, )
        html += '</form>'
        return html

    def get_request_url(self):
        return settings.get_request_url(self.transaction.bank_name)

    def get_encoding(self):
        return 'UTF-8'

    def get_redirect_response_html(self):
        return render_to_string("thorbanks/request.html", {'form': self})

    def get_redirect_response(self):
        return HttpResponse(self.get_redirect_response_html())


class PaymentRequest(PaymentRequestBase):
    """ Creates payment request and Transaction object.

    It also acts as a facade through which transaction object and HTTP redirect can be retrieved.
    """
    VK_SERVICE = forms.CharField(widget=forms.HiddenInput())
    VK_VERSION = forms.CharField(widget=forms.HiddenInput())
    VK_SND_ID = forms.CharField(widget=forms.HiddenInput())
    VK_STAMP = forms.CharField(widget=forms.HiddenInput())
    VK_AMOUNT = forms.CharField(widget=forms.HiddenInput())
    VK_CURR = forms.CharField(widget=forms.HiddenInput())
    VK_REF = forms.CharField(widget=forms.HiddenInput(), required=False)
    VK_MSG = forms.CharField(widget=forms.HiddenInput())
    VK_RETURN = forms.CharField(widget=forms.HiddenInput())
    VK_CANCEL = forms.CharField(widget=forms.HiddenInput())
    VK_DATETIME = forms.CharField(widget=forms.HiddenInput())
    VK_MAC = forms.CharField(widget=forms.HiddenInput(), required=False)
    VK_LANG = forms.CharField(widget=forms.HiddenInput())
    VK_ENCODING = forms.CharField(widget=forms.HiddenInput())

    @classmethod
    def prepare(cls, transaction, url, language='EST'):
        assert language in ['EST', 'ENG', 'RUS']

        return {
            'VK_SERVICE': '1012',
            'VK_VERSION': '008',
            'VK_ENCODING': 'UTF-8',

            'VK_DATETIME': transaction.created.strftime('%Y-%m-%dT%H:%M:%S%z'),
            'VK_RETURN': url,
            'VK_CANCEL': url,
            'VK_LANG': language,
            'VK_SND_ID': settings.get_client_id(transaction.bank_name),

            'VK_STAMP': transaction.pk,
            'VK_REF': calculate_731_checksum(transaction.pk) if get_send_ref(transaction.bank_name) else '',

            'VK_AMOUNT': transaction.amount,
            'VK_CURR': transaction.currency,
            'VK_MSG': transaction.message,
        }

    def finalize(self):
        self.data['VK_MAC'] = create_signature(self.cleaned_data, self.transaction.bank_name)


class PaymentFormMixin(object):
    """
    Base form mixin that lets the user choose the bank used for payment.
    Note that you MUST add bank_name field to your form class, you can use `get_bank_name_field()` method to get a field
    object for that. E.g.

    ```
    class OrderForm(PaymentFormMixin, forms.Form):
        bank_name = PaymentFormMixin.get_bank_name_field()
        # Add your custom fields and/or inherit from ModelForm instead of Form
    ```
    """
    class BankNameFieldRenderer(RadioFieldRenderer):
        """ Augmented RadioFieldRenderer that lets us use out own CSS class.
        """
        UL_HTML = '<ul class="payment-options clearfix">%s</ul>'
        LI_HTML = '<li>%s</li>'

        def field_render(self, field):
            if 'id' in field.attrs:
                label_for = ' for="%s_%s"' % (field.attrs['id'], field.index)
            else:
                label_for = ''
            choice_label = conditional_escape(force_text(field.choice_label))

            return mark_safe('%s <label%s>%s</label>' % (field.tag(), label_for, choice_label))

        def render(self):
            return mark_safe(self.UL_HTML % '\n'.join(
                [self.LI_HTML % force_text(self.field_render(w)) for w in self]))

    def __init__(self, *args, **kwargs):
        self.banklink_order_overwrite = kwargs.pop('banklink_order_overwrite', dict())
        # None means all banks in the settings are permitted
        payment_methods = kwargs.pop('payment_methods', None)

        super(PaymentFormMixin, self).__init__(*args, **kwargs)

        if not self.fields['bank_name'].required:
            raise NotImplementedError('bank_name field must be required')

        self.fields['bank_name'].choices = self.get_payment_method_choices(payment_methods)
        self.fields['bank_name'].error_messages['required'] = _('You have to select a method of payment.')

    @classmethod
    def get_bank_name_field(cls):
        return forms.ChoiceField(label='', required=True,
                                 widget=forms.RadioSelect(renderer=cls.BankNameFieldRenderer))

    def get_payment_method_choices(self, all_payment_methods):
        payment_choices = []
        label_pattern = """<img alt="%s" src="%s" />"""

        for bank_id, bank_name, image_name, order in settings.get_bank_choices():
            if self.banklink_order_overwrite and bank_id in self.banklink_order_overwrite:
                order = self.banklink_order_overwrite[bank_id]

            if all_payment_methods is None or bank_id in all_payment_methods:
                label = label_pattern % (bank_name, self.get_bank_image_path(image_name))
                payment_choices.append((bank_id, mark_safe(label), order))

        return [(y[0], y[1]) for y in sorted(payment_choices, key=lambda x: x[2])]

    def get_bank_image_path(self, image_name):
        logos_dir = getattr(django_settings, 'BANKLINK_LOGO_PATH', 'img/payment/')
        return '%s%s%s' % (django_settings.STATIC_URL, logos_dir, image_name)

    def get_bank_id(self):
        if not self.cleaned_data:
            return None

        return self.cleaned_data['bank_name']



from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


@python_2_unicode_compatible
class AbstractTransaction(models.Model):
    STATUS_PENDING = 'P'
    STATUS_FAILED = 'F'
    STATUS_COMPLETED = 'C'
    STATUS_CHOICES = (
        (STATUS_PENDING, _('pending')),
        (STATUS_FAILED, _('failed')),
        (STATUS_COMPLETED, _('completed')),
    )

    bank_name = models.CharField(_("bank"), max_length=16)
    description = models.CharField(_("reference description"), max_length=255, help_text=_("reference description"))
    amount = models.FloatField(_("amount"))
    currency = models.CharField(_("currency"), max_length=3)
    created = models.DateTimeField(auto_now_add=True)
    last_modified = models.DateTimeField(auto_now=True)
    status = models.CharField(_("status"), max_length=1, default=STATUS_PENDING, choices=STATUS_CHOICES)
    redirect_after_success = models.CharField(max_length=255, editable=False)
    redirect_on_failure = models.CharField(max_length=255, editable=False)

    def __str__(self):
        return "Transaction %s - %s %.2f from %s [%s]" % \
               (self.id, self.currency, self.amount, self.bank_name, self.get_status_display())

    class Meta:
        abstract = True
        app_label = 'thorbanks'
        verbose_name = _("transaction")
        ordering = ['-last_modified']


@python_2_unicode_compatible
class AbstractAuthentication(models.Model):
    STATUS_PENDING = 'P'
    STATUS_FAILED = 'F'
    STATUS_COMPLETED = 'C'
    STATUS_CHOICES = (
        (STATUS_PENDING, _('Pending')),
        (STATUS_FAILED, _('Failed')),
        (STATUS_COMPLETED, _('Completed')),
    )

    status = models.CharField(_("Status"), max_length=1, default=STATUS_PENDING, choices=STATUS_CHOICES)
    bank_name = models.CharField(_("Bank"), max_length=16)
    created = models.DateTimeField(auto_now_add=True)
    last_modified = models.DateTimeField(auto_now=True)
    redirect_after_success = models.CharField(max_length=255, editable=False)
    redirect_on_failure = models.CharField(max_length=255, editable=False)
    raw_response = models.TextField(blank=True)

    def __str__(self):
        return "Authentication %s - %s [%s]" % (self.id, self.bank_name, self.get_status_display())  # pragma no cover

    class Meta:
        abstract = True
        app_label = 'thorbanks'
        verbose_name = _("Authentication")
        ordering = ['-last_modified']



from __future__ import unicode_literals

from django.contrib import admin

from thorbanks import settings


if not settings.manual_models('Transaction'):
    admin.site.register(settings.get_model('Transaction'))


if not settings.manual_models('Authentication'):
    admin.site.register(settings.get_model('Authentication'))



from __future__ import unicode_literals

from thorbanks import settings
from thorbanks.abstract_models import AbstractTransaction, AbstractAuthentication
from thorbanks.loading import is_model_registered


# list of all classes in this module
__all__ = []


if not settings.manual_models('Transaction'):
    if not is_model_registered('thorbanks', 'Transaction'):
        class Transaction(AbstractTransaction):
            pass

        __all__.append('Transaction')


if not settings.manual_models('Authentication'):
    if not is_model_registered('thorbanks', 'Authentication'):
        class Authentication(AbstractAuthentication):
            pass

        __all__.append('Authentication')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import random
from base64 import b64encode
from Crypto.Hash import SHA

import pytest

from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.utils import six

try:
    from django.utils.encoding import force_bytes

except ImportError:
    from django.utils.encoding import smart_str as force_bytes

from tests.utils import assert_ex_msg
from thorbanks import settings as th_settings
from thorbanks.forms import PaymentRequest
from thorbanks.utils import pingback_url, request_digest, get_pkey


def get_banklink_config(bank_name=None, printable_name=None):
    import os

    base_dir = os.path.join(os.path.dirname(__file__), '..')

    return {
        bank_name or 'swedbank': {
            'PRINTABLE_NAME': printable_name or 'Swedbank',
            'REQUEST_URL': 'http://banks.maximum.thorgate.eu/banklink/swedbank-common',
            'CLIENT_ID': 'uid100052',
            'BANK_ID': 'HP',
            'PRIVATE_KEY': os.path.join(base_dir, 'certs', 'swed_key.pem'),
            'PUBLIC_KEY': os.path.join(base_dir, 'certs', 'swed_pub.pem'),
            'TYPE': 'banklink',
            'IMAGE_PATH': 'swedbank.png',
            'ORDER': 1,
            'SEND_REF': True,
        },
        'seb': {
            'PRINTABLE_NAME': 'SEB',
            'REQUEST_URL': 'http://banks.maximum.thorgate.eu/banklink/seb-common',
            'CLIENT_ID': 'uid100036',
            'BANK_ID': 'EYP',
            'PRIVATE_KEY': os.path.join(base_dir, 'certs', 'seb_key.pem'),
            'PUBLIC_KEY': os.path.join(base_dir, 'certs', 'seb_pub.pem'),
            'DIGEST_COUNTS_BYTES': True,
            'TYPE': 'banklink',
            'IMAGE_PATH': 'seb.png',
            'ORDER': 2,
            'SEND_REF': False,
        },
        'lhv': {
            'PRINTABLE_NAME': 'LHV',
            'REQUEST_URL': 'http://banks.maximum.thorgate.eu/banklink/lhv-common',
            'CLIENT_ID': 'uid100049',
            'BANK_ID': 'LHV',
            'PRIVATE_KEY': os.path.join(base_dir, 'certs', 'lhv_key.pem'),
            'PUBLIC_KEY': os.path.join(base_dir, 'certs', 'lhv_pub.pem'),
            'TYPE': 'banklink',
            'IMAGE_PATH': 'lhv.png',
            'ORDER': 3,
        },
        'danske': {
            'PRINTABLE_NAME': 'Danske',
            'REQUEST_URL': 'http://banks.maximum.thorgate.eu/banklink/sampo-common',
            'CLIENT_ID': 'uid100010',
            'BANK_ID': 'SAMPOPANK',
            'PRIVATE_KEY': os.path.join(base_dir, 'certs', 'danske_key.pem'),
            'PUBLIC_KEY': os.path.join(base_dir, 'certs', 'danske_pub.pem'),
            'TYPE': 'banklink',
            'IMAGE_PATH': 'danske.png',
            'ORDER': 4,
        },
    }


def test_no_settings_raises_exception(settings):
    settings.BANKLINKS = None

    with pytest.raises(ImproperlyConfigured) as e:
        th_settings.configure()

    assert_ex_msg(e, "BANKLINKS not found in settings")


def test_banklinks_is_not_dict(settings):
    settings.BANKLINKS = True

    with pytest.raises(ImproperlyConfigured) as e:
        th_settings.configure()

    assert_ex_msg(e, "settings.BANKLINKS must be dict")


def test_banklink_name_too_long(settings):
    settings.BANKLINKS = get_banklink_config(bank_name="ABCDEFGHIJKLMNOPS")

    with pytest.raises(ImproperlyConfigured) as e:
        th_settings.configure()

    assert_ex_msg(e, "Bank's name must be at most 16 characters (ABCDEFGHIJKLMNOPS)")


def test_banklink_data_not_dict(settings):
    settings.BANKLINKS = {'swedbank': None}

    with pytest.raises(ImproperlyConfigured) as e:
        th_settings.configure()

    assert_ex_msg(e, "Each bank in settings.BANKLINKS must correspond to dict with settings of that bank")


def test_deprecated_keys(settings):
    settings.BANKLINKS = get_banklink_config()
    settings.BANKLINKS['swedbank']['SND_ID'] = '123'

    with pytest.raises(ImproperlyConfigured) as e:
        th_settings.configure()

    assert_ex_msg(e, "SND_ID is removed. Use CLIENT_ID instead swedbank")


@pytest.mark.parametrize("missing_key", ['PRIVATE_KEY', 'PUBLIC_KEY', 'CLIENT_ID', 'REQUEST_URL', 'BANK_ID'])
def test_missing_keys(settings, missing_key):
    settings.BANKLINKS = get_banklink_config()
    del settings.BANKLINKS['swedbank'][missing_key]

    with pytest.raises(ImproperlyConfigured) as e:
        th_settings.configure()

    assert_ex_msg(e, "%s not found in settings for bank swedbank" % missing_key)


@pytest.mark.parametrize("key_type", ['private', 'public'])
def test_no_key_file(settings, key_type):
    file_name = "rand_name_%d.key" % random.randint(1, 999999)

    settings.BANKLINKS = get_banklink_config()
    settings.BANKLINKS['swedbank']['%s_KEY' % key_type.upper()] = file_name

    with pytest.raises(ImproperlyConfigured) as e:
        th_settings.configure()

    assert_ex_msg(e, "%s%s key file %s for bank swedbank does not exist." % (key_type[0].upper(), key_type[1:].lower(), file_name))


def test_getters(settings):
    conf = get_banklink_config()
    settings.BANKLINKS = conf
    th_settings.configure()

    assert th_settings.get_private_key("swedbank") == conf["swedbank"]["PRIVATE_KEY"]
    assert th_settings.get_public_key("swedbank") == conf["swedbank"]["PUBLIC_KEY"]
    assert th_settings.get_client_id("swedbank") == conf["swedbank"]["CLIENT_ID"]
    assert th_settings.get_request_url("swedbank") == conf["swedbank"]["REQUEST_URL"]
    assert th_settings.get_link_type("swedbank") == conf["swedbank"]["TYPE"]

    assert th_settings.get_send_ref("swedbank") == conf["swedbank"]["SEND_REF"]
    assert th_settings.get_send_ref("seb") == conf["seb"]["SEND_REF"]
    assert th_settings.get_send_ref("danske") is True

    the_list = th_settings.get_bank_choices()
    assert isinstance(the_list, list)

    for item in the_list:
        assert isinstance(item, tuple)
        assert len(item) == 4
        assert item[0] in conf.keys()

        assert item[1] == conf[item[0]]['PRINTABLE_NAME']
        assert item[2] == conf[item[0]]['IMAGE_PATH']
        assert item[3] == conf[item[0]].get('ORDER', 99)


def test_transaction_model():
    trans = th_settings.get_model('Transaction')(pk=1, currency='EUR', amount=15.67, bank_name='myBank')

    assert trans.__str__() == "Transaction 1 - EUR 15.67 from myBank [pending]"


def test_pingback_with_request(rf):
    request = rf.get('/test', HTTP_HOST='127.0.0.1')

    ping_back = pingback_url(request=request)

    assert ping_back == 'http://%s%s' % (request.META['HTTP_HOST'], reverse('thorbanks_response'))


@pytest.mark.django_db
def test_payment_result(settings):
    settings.BANKLINKS = get_banklink_config()
    th_settings.configure()

    redirect_to = "http://example.com"
    ping_back = pingback_url(base_url=redirect_to)

    assert ping_back == redirect_to + reverse('thorbanks_response')

    request = PaymentRequest(
        bank_name="swedbank",
        amount=13.99,
        currency='EUR',
        redirect_to=redirect_to,
        redirect_on_failure=redirect_to,
        message="My cool payment",
        url=ping_back,
    )

    assert request.transaction.bank_name == "swedbank"
    assert request.transaction.amount == 13.99
    assert request.transaction.currency == 'EUR'
    assert request.transaction.redirect_after_success == redirect_to
    assert request.transaction.redirect_on_failure == redirect_to
    assert request['VK_RETURN'].value() == ping_back

    expected_digest = force_bytes(
        '0041012003008009uid100052001100513.99003EUR00213015My cool payment044http://example'
        '.com/banks/thorbanks_response/044http://example.com/banks/thorbanks_response/024%s' %
        request.transaction.created.strftime('%Y-%m-%dT%H:%M:%S%z')
    )

    assert request_digest(request.cleaned_data, request.transaction.bank_name) == expected_digest
    assert request['VK_MAC'].value() == b64encode(get_pkey('swedbank').sign(SHA.new(expected_digest)))


@pytest.mark.django_db
def test_payment_form_html_decode_error():
    form = PaymentRequest(
        bank_name="swedbank",
        amount=20.40,
        currency='EUR',
        redirect_to="http://example.com",
        redirect_on_failure="http://example.com",
        message="Pangalink: 12345 (müsli)",
        url="http://example.com",
    )

    assert isinstance(form.submit_button(), six.text_type)
    assert isinstance(form.redirect_html(), six.text_type)
    assert isinstance(form.as_html(), six.text_type)  # TODO: Remove once we drop the method



from __future__ import unicode_literals

import logging

from django.utils.crypto import constant_time_compare
from django.utils.decorators import method_decorator
from django.views.generic import View
from django.http import HttpResponseRedirect
from django.http import HttpResponse, QueryDict
from django.shortcuts import get_object_or_404
from django.views.decorators.csrf import csrf_exempt

from thorbanks import settings
from thorbanks.forms import PaymentRequest, IPizzaAuthRequest, NordeaAuthRequest
from thorbanks.utils import verify_signature, nordea_generate_mac
from thorbanks.signals import transaction_succeeded, auth_succeeded, auth_failed
from thorbanks.signals import transaction_failed


logger = logging.getLogger(__name__)


def get_request_data(request):
    if request.method == 'POST':
        return request.POST
    else:
        return request.GET


class PaymentError(Exception):
    """Generic error class."""


class AuthError(Exception):
    """Generic error class."""


@csrf_exempt
def response(request):
    data = get_request_data(request)
    if 'VK_MAC' not in data:
        raise PaymentError("VK_MAC not in request")

    klass = settings.get_model('Transaction')
    transaction = get_object_or_404(klass, pk=data['VK_STAMP'])

    # Set proper encoding and get the data again (this time in correct encoding)
    request.encoding = 'UTF-8'

    # We have to manually replace POST after changing encoding because of a Django bug (?)
    request.POST = QueryDict(request.body, encoding=request.encoding)
    data = get_request_data(request)

    signature_valid = verify_signature(data, transaction.bank_name, data['VK_MAC'], response=True)
    if not signature_valid:
        raise PaymentError("Invalid signature. ")

    if data['VK_SERVICE'] == '1111':
        if transaction.status == klass.STATUS_PENDING:
            # Mark purchase as complete
            transaction.status = klass.STATUS_COMPLETED
            transaction.save()
            transaction_succeeded.send(klass, transaction=transaction)
    elif data['VK_SERVICE'] == '1911':
        if transaction.status == klass.STATUS_PENDING:
            # Mark purchase as failed
            transaction.status = klass.STATUS_FAILED
            transaction.save()
            transaction_failed.send(klass, transaction=transaction)
    else:
        logging.critical("thorbanks.views.response(): Got invalid VK_SERVICE code from bank: %s (transaction %s)",
                         data['VK_SERVICE'], data['VK_STAMP'])

        raise PaymentError("Bank sent confirmation with invalid VK_SERVICE!")

    if data['VK_AUTO'] == 'Y':
        # This is automatic pingback from the bank - send simple 200 response.
        return HttpResponse("request handled")

    else:
        # This request is from the user after being redirected from the bank to our server. Redirect her further.
        if data['VK_SERVICE'] == '1111':
            url = transaction.redirect_after_success
        else:
            url = transaction.redirect_on_failure

        return HttpResponseRedirect(url)


def create_payment_request(bank_name, message, amount, currency, pingback_url, redirect_to, redirect_on_failure):
    return PaymentRequest(
        bank_name=bank_name,
        amount=amount,
        currency=currency,
        redirect_to=redirect_to,
        redirect_on_failure=redirect_on_failure,
        message=message,
        url=pingback_url,
    )


def create_auth_request(request, bank_name, response_url, redirect_to=None):
    request_form_classes = {
        'ipizza': IPizzaAuthRequest,
        'nordea': NordeaAuthRequest,
    }
    form_cls = request_form_classes.get(settings.get_link_protocol(bank_name))
    if form_cls is None:
        return None

    return form_cls(
        bank_name=bank_name,
        response_url=response_url,
        redirect_to=redirect_to or response_url,
    )


class AuthResponseView(View):
    def __init__(self, **kwargs):
        self.data = None
        self.url = None
        self.auth = None

        super(AuthResponseView, self).__init__(**kwargs)

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        self.data = get_request_data(request)

        if 'VK_MAC' in self.data:
            self.handle_ipizza(request)
        elif 'B02K_MAC' in self.data:
            self.handle_nordea(request)
        else:
            raise AuthError("Invalid request (unknown protocol)")

        return super(AuthResponseView, self).dispatch(request, *args, **kwargs)

    def handle_ipizza(self, request):
        klass = settings.get_model('Authentication')
        auth = get_object_or_404(klass, pk=self.data.get('VK_NONCE', None))

        # Set proper encoding and get the data again (this time in correct encoding)
        request.encoding = 'UTF-8'

        # We have to manually replace POST after changing encoding because of a Django bug (?)
        request.POST = QueryDict(request.body, encoding=request.encoding)
        self.data = get_request_data(request)

        signature_valid = verify_signature(self.data, auth.bank_name, self.data['VK_MAC'], auth=True, response=True)
        if not signature_valid:
            raise AuthError("Invalid signature. ")

        self.data = self.data.dict()

        if self.data['VK_SERVICE'] == '3013':
            if auth.status == klass.STATUS_PENDING:
                # Mark auth as complete
                auth.raw_response = self.data
                auth.status = klass.STATUS_COMPLETED
                auth.save()
                auth_succeeded.send(klass, auth=auth)

        else:
            if auth.status == klass.STATUS_PENDING:
                # Mark auth as failed
                auth.raw_response = self.data
                auth.status = klass.STATUS_FAILED
                auth.save()
                auth_failed.send(klass, auth=auth)

        # This request is from the user after being redirected from the bank to our server. Redirect her further.
        if self.data['VK_SERVICE'] == '3013':
            self.url = auth.redirect_after_success
        else:
            self.url = auth.redirect_on_failure

        self.auth = auth

        self.data['name'] = self.data.get('VK_USER_NAME')
        self.data['person_code'] = self.data.get('VK_USER_ID')

    def handle_nordea(self, request):
        klass = settings.get_model('Authentication')
        auth = get_object_or_404(klass, pk=self.data.get('B02K_STAMP', None))

        # Set proper encoding (Nordea supports only ISO-8859-1) and get the data again (this time in correct encoding)
        request.encoding = 'ISO-8859-1'

        # We have to manually replace GET and POST after changing encoding because of a Django bug (?)
        request.POST = QueryDict(request.body, encoding=request.encoding)
        request.GET = QueryDict(request.META.get('QUERY_STRING', ''), encoding=request.encoding)
        self.data = get_request_data(request)
        self.data = self.data.dict()

        # Calculate message's MAC and compare it to the one sent to us
        link_config = settings.LINKS[auth.bank_name]
        mac_key = link_config['MAC_KEY']
        mac_token_fields = (
            'B02K_VERS',
            'B02K_TIMESTMP',
            'B02K_IDNBR',
            'B02K_STAMP',
            'B02K_CUSTNAME',
            'B02K_KEYVERS',
            'B02K_ALG',
            'B02K_CUSTID',
            'B02K_CUSTTYPE'
        )
        mac = nordea_generate_mac(self.data, mac_token_fields, mac_key)
        if not constant_time_compare(mac, self.data['B02K_MAC']):
            raise AuthError("Invalid signature. ")

        # This is the same as what we send in the auth request and we only support 0002 ATM
        assert self.data['B02K_VERS'] == '0002'

        # Nordea only sends success messages
        if auth.status == klass.STATUS_PENDING:
            # Mark auth as complete
            auth.raw_response = self.data
            auth.status = klass.STATUS_COMPLETED
            auth.save()
            auth_succeeded.send(klass, auth=auth)

        self.url = auth.redirect_after_success

        self.auth = auth

        self.data['name'] = self.data.get('B02K_CUSTNAME')
        self.data['person_code'] = self.data.get('B02K_CUSTID')

    def get(self, request, *args, **kwargs):
        return HttpResponseRedirect(self.url)

    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)



from __future__ import unicode_literals

import logging

from django.db import models
from django.dispatch.dispatcher import receiver

from thorbanks.signals import transaction_succeeded, transaction_failed


class Order(models.Model):
    amount = models.FloatField()

    transaction = models.OneToOneField('thorbanks.Transaction', null=True)
    is_paid = models.BooleanField(default=False)

    def complete(self):
        self.is_paid = True
        self.save()


@receiver(transaction_succeeded)
def banklink_success_callback(sender, transaction, **kwargs):
    """ Gets called when we have a confirmation from the bank that this transaction was completed.
         The logic should only be run on the first callback.
    """
    logging.info("Banklink transfer callback with transaction %s" % transaction.id)
    try:
        # Mark the order as paid
        if transaction.order:
            transaction.order.complete()

        logging.info("Transaction %d purchase completed", transaction.id)

    except Exception:
        # *WARNING*:
        # This is a very bad exception. It means the user gave us money, but for some reason we
        # couldn't give him the items(complete the purchase)
        # If this happens contact has to be made with the client to resolve the issue.
        logging.exception("Transaction %d purchase failed." % transaction.id)


@receiver(transaction_failed)
def banklink_failed_callback(sender, transaction, **kwargs):
    logging.warning("Transaction %d purchase failed." % transaction.id)



from __future__ import unicode_literals

from functools import reduce
from base64 import b64encode, b64decode
import hashlib

from django.core.urlresolvers import reverse
from django.utils.encoding import force_text

from Crypto.Hash import SHA
from Crypto.Signature import PKCS1_v1_5
from Crypto.PublicKey import RSA

from thorbanks import settings


IPIZZA_REQUEST_ORDER = {
    '3012': (
        'VK_SERVICE', 'VK_VERSION', 'VK_USER',
        'VK_DATETIME', 'VK_SND_ID', 'VK_REC_ID',
        'VK_USER_NAME', 'VK_USER_ID', 'VK_COUNTRY',
        'VK_OTHER', 'VK_TOKEN', 'VK_RID',
    ),
    '3013': (
        'VK_SERVICE', 'VK_VERSION', 'VK_DATETIME',
        'VK_SND_ID', 'VK_REC_ID', 'VK_NONCE',
        'VK_USER_NAME', 'VK_USER_ID', 'VK_COUNTRY',
        'VK_OTHER', 'VK_TOKEN', 'VK_RID',
    ),
    '4011': (
        'VK_SERVICE', 'VK_VERSION', 'VK_SND_ID',
        'VK_REPLY', 'VK_RETURN', 'VK_DATETIME',
        'VK_RID',
    ),
    '4012': (
        'VK_SERVICE', 'VK_VERSION', 'VK_SND_ID',
        'VK_REC_ID', 'VK_NONCE', 'VK_RETURN',
        'VK_DATETIME', 'VK_RID',
    ),
}


def get_ordered_request(request, auth=False, response=False):
    def append_if_exists(target, source, the_value):
        if the_value in source:
            target.append(source[the_value])
        return target

    if not auth:
        if response:
            expected_values = (
                'VK_SERVICE', 'VK_VERSION', 'VK_SND_ID',
                'VK_REC_ID', 'VK_STAMP', 'VK_T_NO',

                'VK_AMOUNT', 'VK_CURR',

                'VK_REC_ACC', 'VK_REC_NAME',

                'VK_SND_ACC', 'VK_SND_NAME',

                'VK_REF', 'VK_MSG', 'VK_T_DATETIME',
            )

        else:
            expected_values = (
                'VK_SERVICE', 'VK_VERSION', 'VK_SND_ID',
                'VK_STAMP', 'VK_AMOUNT', 'VK_CURR',
                'VK_REF', 'VK_MSG', 'VK_RETURN',
                'VK_CANCEL', 'VK_DATETIME',
            )

    else:
        if response:
            expected_values = IPIZZA_REQUEST_ORDER['3013']

        else:
            expected_values = IPIZZA_REQUEST_ORDER['4012']

    ordered_request = []
    for value in expected_values:
        ordered_request = append_if_exists(ordered_request, request, value)
    return ordered_request


def request_digest(request, bank_name, auth=False, response=False):
    """
        return request digest in Banklink signature form (see docs for format)
    """
    request = get_ordered_request(request, auth=auth, response=response)
    digest = ''
    for value in request:
        value_len = len(value)
        digest += force_text(value_len).rjust(3, '0')
        digest += force_text(value)
    return digest.encode('UTF-8')


def get_pkey(bank_name):
    with open(settings.get_private_key(bank_name)) as handle:
        private_key = RSA.importKey(handle.read())
        handle.close()

    return PKCS1_v1_5.new(private_key)


def create_signature(request, bank_name, auth=False):
    """
        sign BankLink request in dict format with private_key
    """
    digest = request_digest(request, bank_name, auth=auth)
    key_binary = get_pkey(bank_name).sign(SHA.new(digest))

    return b64encode(key_binary)


def verify_signature(request, bank_name, signature, auth=False, response=False):
    """
        verify BankLink reply signature
    """
    signature = force_text(signature)

    with open(settings.get_public_key(bank_name)) as handle:
        public_key = RSA.importKey(handle.read())
        handle.close()

    the_key = PKCS1_v1_5.new(public_key)
    digest = request_digest(request, bank_name, auth=auth, response=response)

    return the_key.verify(SHA.new(digest), b64decode(signature))


def nordea_generate_mac(data, fields, mac_key):
    mac_tokens = [data.get(field, '') for field in fields] + [mac_key]
    mac_tokens = [token.encode('iso-8859-1') for token in mac_tokens]
    mac_data = b'&'.join(mac_tokens) + b'&'

    h = hashlib.md5(mac_data)
    return h.hexdigest().upper()


def weight_generator():
    """ Used for weight generation by calculate_731_checksum
    """
    while True:
        yield 7
        yield 3
        yield 1


def calculate_731_checksum(number):
    # Check if number is integer, then cast to string
    number = str(int(number))[::-1]
    gen = weight_generator()
    weight_sum = reduce(lambda x, y: x + int(y) * next(gen), number, 0)
    checksum = (10 - weight_sum % 10) % 10
    return int(number[::-1] + str(checksum))


def pingback_url(request=None, base_url=None):
    assert request or base_url

    if request:
        base_url = '%s://%s' % ('https' if request.is_secure() else 'http', request.META['HTTP_HOST'])

    return '%s%s' % (base_url, reverse('thorbanks_response'))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import random
import pytest
import time

from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions

from django.core.urlresolvers import reverse

from tests.utils import select_radio, click, IPIZZA_BANKS, ready, set_input_text, assert_no_errors, IS_TRAVIS, TIMEOUT


@pytest.mark.parametrize("bank_name", IPIZZA_BANKS)
@pytest.mark.parametrize("send_ref", [True, False])
@pytest.mark.django_db
@pytest.mark.timeout(TIMEOUT)
def test_payment_flow(bank_name, send_ref, live_server, selenium):
    from shop.models import Order
    from thorbanks.models import Transaction

    assert bank_name in IPIZZA_BANKS

    if IS_TRAVIS:
        time.sleep(1 + random.uniform(0.5, 2.3))  # Give server time to start up

    pay_url = '%s%s?send_ref=%d' % (live_server.url, reverse('payment'), 1 if send_ref else 0)

    # Get the order page
    selenium.get(pay_url)
    selenium.implicitly_wait(3)  # 3 seconds of grace time when there might be lag

    # wait for the page to load
    WebDriverWait(selenium, 10).until(
        expected_conditions.title_contains('Thorbanks')
    )

    # Also wait for styles
    WebDriverWait(selenium, 10).until(ready)

    # Set amount to a random value
    amount = round(random.uniform(0.1, 5000), 5)
    set_input_text('amount', str(amount), selenium)

    # Select correct radio
    select_radio('bank_name', bank_name, selenium)

    # Click Pay
    click('input[type="submit"]', selenium)

    # Wait for the banks test page to load
    WebDriverWait(selenium, 10).until(
        expected_conditions.title_contains('Pangalink-net')
    )

    # Also wait for styles
    WebDriverWait(selenium, 10).until(ready)

    # Check that there are no errors
    assert_no_errors(selenium)

    # Set custom value for name
    set_input_text('senderName', 'Tõõgera Leõpäöldi', selenium)

    # Check that the amount is correct
    elem = selenium.find_element_by_css_selector('form table tr:last-child td:last-child')
    assert elem
    assert elem.text.replace(' ', '') == ('%.2f€' % amount).replace('.', ',')

    # Click submit
    click('button[type="submit"]', selenium)

    # Wait until return button is clickable
    WebDriverWait(selenium, 10).until(
        expected_conditions.element_to_be_clickable((By.CSS_SELECTOR, '[data-button="return"]'))
    )

    # Click the return button
    click('[data-button="return"]', selenium)

    # Wait until we are redirected back to example app
    WebDriverWait(selenium, 10).until(
        expected_conditions.title_contains('Thorbanks')
    )

    assert 'success' in selenium.current_url
    url_parts = [x for x in selenium.current_url.split('/') if x]

    assert len(url_parts) == 5
    order_id = url_parts[-2]
    assert order_id and all([x.isdigit() for x in order_id])

    # Get the transaction from db
    order = Order.objects.get(pk=order_id)

    assert order and order.pk
    assert round(order.amount, 2) == round(amount, 2)
    assert order.is_paid

    assert order.transaction_id

    assert order.transaction.bank_name == bank_name
    assert order.transaction.amount == round(amount, 2)
    assert order.transaction.currency == 'EUR'
    assert order.transaction.status == Transaction.STATUS_COMPLETED



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import random
import pytest
import time

from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions

from django.core.urlresolvers import reverse

from tests.utils import select_radio, click, IPIZZA_BANKS, ready, set_input_text, assert_no_errors, IS_TRAVIS, TIMEOUT


@pytest.mark.parametrize("bank_name", IPIZZA_BANKS)
@pytest.mark.django_db
@pytest.mark.timeout(TIMEOUT)
def test_auth_flow(bank_name, live_server, selenium):
    if IS_TRAVIS:
        time.sleep(1 + random.uniform(0.5, 2.3))  # Give server time to start up

    auth_url = '%s%s' % (live_server.url, reverse('auth'))

    # Get the auth page
    selenium.get(auth_url)
    selenium.implicitly_wait(3)  # 3 seconds of grace time when there might be lag

    # wait for the page to load
    WebDriverWait(selenium, 10).until(
        expected_conditions.title_contains('Thorbanks')
    )

    # Also wait for styles
    WebDriverWait(selenium, 10).until(ready)

    # Select correct radio
    select_radio('bank_name', bank_name, selenium)

    # Click Authenticate
    click('input[type="submit"]', selenium)

    # Wait for the banks test page to load
    WebDriverWait(selenium, 10).until(
        expected_conditions.title_contains('Pangalink-net')
    )

    # Also wait for styles
    WebDriverWait(selenium, 10).until(ready)

    # Check that there are no errors
    assert_no_errors(selenium)

    # Set custom values for name and personal id
    set_input_text('authUserName', 'Tõõgera Leõpäöldi', selenium)
    set_input_text('authUserId', '51001091072', selenium)

    # Click submit
    click('button[type="submit"]', selenium)

    # Wait until return button is clickable
    WebDriverWait(selenium, 10).until(
        expected_conditions.element_to_be_clickable((By.CSS_SELECTOR, '[data-button="return"]'))
    )

    # Click the return button
    click('[data-button="return"]', selenium)

    # Wait until we are redirected back to example app
    WebDriverWait(selenium, 10).until(
        expected_conditions.title_contains('Thorbanks')
    )

    # Assert correct result
    result = selenium.find_element_by_css_selector('[data-result]')
    assert result and result.text == 'success'

    # Assert name
    result = selenium.find_element_by_css_selector('[data-name]')
    assert result and result.text == 'Tõõgera Leõpäöldi'

    # Assert person code
    result = selenium.find_element_by_css_selector('[data-code]')
    assert result and result.text == '51001091072'



from __future__ import unicode_literals

import sys

from django import http
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.views.debug import ExceptionReporter
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import TemplateView
from django.views.generic.detail import DetailView
from django.views.generic.edit import CreateView, FormView
from thorbanks.settings import LINKS

from thorbanks.views import create_payment_request, create_auth_request, AuthResponseView, AuthError
from thorbanks.utils import pingback_url

from shop.forms import AuthForm, OrderForm
from shop.models import Order


class FrontpageView(TemplateView):
    template_name = 'frontpage.html'


class PaymentView(CreateView):
    template_name = 'payment/index.html'
    form_class = OrderForm

    def form_valid(self, form):
        # Create new Order object, with null transaction
        self.object = form.save()

        # Get urls of the order's success/failure pages
        redirect_url = reverse('order-success', kwargs={'pk': self.object.id})
        redirect_on_failure_url = reverse('order-failed', kwargs={'pk': self.object.id})

        # Allow overwriting of send_ref value via url param (this is for unittests)
        # WARNING: Don't use this in production code
        old_val = LINKS[form.cleaned_data['bank_name']].get('SEND_REF', True)
        LINKS[form.cleaned_data['bank_name']]['SEND_REF'] = self.request.GET.get('send_ref', '1') == '1'

        # Create new payment request
        payment = create_payment_request(bank_name=form.cleaned_data['bank_name'], message="My cool payment",
                                         amount=form.cleaned_data['amount'], currency='EUR',
                                         pingback_url=pingback_url(self.request),
                                         redirect_on_failure=redirect_on_failure_url,
                                         redirect_to=redirect_url)

        # restore old SEND_REF value
        LINKS[form.cleaned_data['bank_name']]['SEND_REF'] = old_val

        # Attach the pending transaction object to the Order object
        self.object.transaction = payment.transaction
        self.object.save()

        # Finally, return the HTTP response which redirects the user to the bank
        return payment.get_redirect_response()


class PaymentSuccess(DetailView):
    model = Order
    template_name = 'payment/success.html'


class PaymentFailed(DetailView):
    model = Order
    template_name = 'payment/failed.html'


class AuthenticationView(FormView):
    template_name = 'auth/index.html'
    form_class = AuthForm

    def form_valid(self, form):
        # Get urls of the order's success/failure pages
        redirect_url = self.request.build_absolute_uri(reverse('auth-complete'))

        # Create new auth request
        bank_name = form.cleaned_data['bank_name']
        auth_form = create_auth_request(
            request=self.request,
            bank_name=bank_name,
            response_url=redirect_url,
        )

        # Finally, return the HTTP response which redirects the user to the bank
        return HttpResponse(auth_form.redirect_html())


class AuthenticationCompleteView(AuthResponseView, TemplateView):
    template_name = 'auth/success.html'

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        try:
            return super(AuthenticationCompleteView, self).dispatch(request, *args, **kwargs)

        except AuthError:
            return self.get(request, *args, **kwargs)

    def validate(self, request):
        if self.auth is None or self.data is None:
            return None

        # Check if the person code is present
        if 'person_code' not in self.data:
            return None

        return self.data

    def get(self, request, *args, **kwargs):
        data = self.validate(request)

        if data is None:
            self.template_name = 'auth/failed.html'
        else:
            self.template_name = 'auth/success.html'

        return self.render_to_response(data)


def show_server_error(request):
    """
    500 error handler to show Django default 500 template
    with nice error information and traceback.
    Useful in testing, if you can't set DEBUG=True.

    Templates: `500.html`
    Context: sys.exc_info() results
     """
    exc_type, exc_value, exc_traceback = sys.exc_info()
    error = ExceptionReporter(request, exc_type, exc_value, exc_traceback)
    return http.HttpResponseServerError(error.get_traceback_html())



from __future__ import unicode_literals

from django.core.exceptions import ImproperlyConfigured

try:
    from django.utils.module_loading import import_string as base_import_string

except ImportError:
    try:
        from django.utils.module_loading import import_by_path as base_import_string

    except ImportError:
        from django.utils.importlib import import_module as base_import_string

try:
    from django.apps import apps

    get_model = apps.get_model
    get_registered_model = apps.get_registered_model

except ImportError:
    from django.db.models import get_model as django_get_model

    def get_model(app_label, model_name=None):
        if model_name is None:
            app_label, model_name = app_label.split('.')
        model = django_get_model(app_label, model_name)
        if not model:
            raise LookupError
        return model

    def get_registered_model(app_label, model_name):
        model = django_get_model(app_label, model_name, seed_cache=False, only_installed=False)
        if not model:
            raise LookupError
        return model


def is_model_registered(app_label, model_name):
    """ Checks whether a given model is registered.
    """
    try:
        get_registered_model(app_label, model_name)
    except LookupError:
        return False
    else:
        return True


def validate_import_string(dotted_path):
    try:
        module_path, class_name = dotted_path.rsplit('.', 1)

    except ValueError:
        raise ImproperlyConfigured("%s doesn't look like a module path" % dotted_path)

    return module_path, class_name


def import_string(dotted_path):
    return base_import_string(dotted_path)



from __future__ import unicode_literals

from django.conf.urls import url, include

from shop.views import (FrontpageView, PaymentView, PaymentSuccess, PaymentFailed,
                        AuthenticationView, AuthenticationCompleteView)


urlpatterns = [
    url(r'^$', FrontpageView.as_view(), name='frontpage'),

    url(r'^order/$', PaymentView.as_view(), name='payment'),
    url(r'^order/(?P<pk>\d+)/success/$', PaymentSuccess.as_view(), name='order-success'),
    url(r'^order/(?P<pk>\d+)/failed/$', PaymentFailed.as_view(), name='order-failed'),

    url(r'^auth/$', AuthenticationView.as_view(), name='auth'),
    url(r'^auth/complete/$', AuthenticationCompleteView.as_view(), name='auth-complete'),

    url(r'^banks/', include('thorbanks.urls')),
]

handler500 = 'shop.views.show_server_error'



from __future__ import unicode_literals

from django import forms

from thorbanks.forms import PaymentFormMixin

from shop.models import Order


class AuthForm(PaymentFormMixin, forms.Form):
    bank_name = PaymentFormMixin.get_bank_name_field()


class OrderForm(PaymentFormMixin, forms.ModelForm):
    class Meta:
        model = Order
        fields = ['amount']

    bank_name = PaymentFormMixin.get_bank_name_field()



from __future__ import unicode_literals

from django.conf.urls import include, url

from django.contrib import admin

admin.autodiscover()


urlpatterns = [
    url(r'^', include('shop.urls')),

    url(r'^admin/', include(admin.site.urls)),
]

handler500 = 'shop.views.show_server_error'



#!/usr/bin/env python

import os
import sys

from setuptools import setup, find_packages

from thorbanks import __version__ as version


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()


with open(os.path.join(os.path.dirname(__file__), 'README.md')) as f:
    readme = f.read()


setup(
    name='django-thorbanks',
    version=version,
    description='`django-thorbanks` provides a Django application for Estonian banklinks.',
    long_description=readme,
    author="Thorgate",
    author_email='info@thorgate.eu',
    url='http://thorgate.eu',
    packages=find_packages(),
    package_data={'thorbanks': [
        'static/img/payment/*',
        'templates/thorbanks/*',
    ]},
    include_package_data=True,
    install_requires=[
        'Django>=1.4,<1.10',
        'PyCrypto',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)



"""
WSGI config for example project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/howto/deployment/wsgi/
"""

import os
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "example.settings")

from django.core.wsgi import get_wsgi_application
application = get_wsgi_application()
